function RESULT = CreateCompactCV_eig(icontrol)
% -------------------------------------------------------------------------
% CreateCompactCV_eig.m
%
% Author         David Humphreys and Peter Harris, NPL
% Created        07-06-2012
% Last amended   18-07-2012   Added RESULT array.
%                25-01-2013   Added numerical output.
%                06-02-2013   Updated.
% Released       08-02-2013
% Last amended   05-05-2013   Modification to how nei is set.
%                10-07-2013   Incorporated path name.
%                10-07-2013   Plotting made optional.
%                15-07-2013   Modification to how nei is set.
%                26-03-2014   Added DC value.
%                31-03-2014   Added test of postive-definiteness.
%                13-04-2014   Alignment made optional.
% Released       30-06-2014
% -------------------------------------------------------------------------
% Create compact representation of covariance matrix for harmonic results
% based on eigendecomposition.
%
% See: Guidance for the use of compact covariance uncertainty software,
%      David Humphreys and Peter Harris, NPL REPORT TQE 10, 2014
% -------------------------------------------------------------------------
% Input:
%    icontrol    2 x 1        Control of alignment and plotting 
%                             (optional): default = [1, 1].
%
% Output:
%    RESULT                   Array storing compact representation of
%                             covariance matrix.
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Set defaults for optional input parameter.
%
  if nargin < 1
     icontrol = [1, 1];
  end
%  
% -------------------------------------------------------------------------
% Load frequency domain data: the first column contains the values of
% frequency (0, f, 2*f, ..., m*f); subsequent columns contain the real
% and imaginary parts (of the frequency domain data) for each waveform.
%
% Note: Use 'xlsread' for csv files and 'importdata' for dat files.
%
  [fname, pname] = uigetfile({'*.csv;*.dat'}, 'Select a DATA file');
  [~, name, ext] = fileparts(fname);
  if strcmp(ext, '.csv')
     M = xlsread([pname, fname]);
  elseif strcmp(ext, '.dat')
     M = importdata([pname, fname]);
  else
     error('File type not recognised')
  end
%
% Number n of frequencies (excluding DC) and number m of waveforms.
%
  n = size(M,1) - 1;
  m = (size(M,2) - 1)/2;
%
% Load frequencies f, DC values and harmonic results z.
%
  GHz = 10^9;
  f   = M(2:n+1,1)/GHz;
  dc  = M(1,2:2:1+2*m);
  z   = M(2:n+1,2:2:1+2*m) + 1i*M(2:n+1,3:2:1+2*m);
  clear M
%
% -------------------------------------------------------------------------
% Pre-processing: alignment to give harmonic results z with mean zref.
%
  if icontrol(1) == 1
     [z, zref] = alignment(z);
  else
     zref = mean(z,2);
  end
%
% -------------------------------------------------------------------------
% Calculate full covariance matrix and store it as three sub-matrices
% corresponding to the Re/Re, Im/Im and Im/Re parts.
%
  [CC, SS, CS] = fullcovariance(z, zref);
  clear z
%
% -------------------------------------------------------------------------
% Calculate the nei largest eigenvalues of the full covariance matrix.
%
  nei = min([2*n-1, m-1]);
  opts.issym = 1;
  E = eigs(@(x)CVfun(x,sparse(CC),sparse(SS),sparse(CS)), ...
     2*n, nei, 'lm', opts);
%
% Decide how many eigenvalues to use to form approximation to covariance
% matrix.
%
  neistart   = 2;
  neiend     = nei;
  [neix, ss] = findneix(E, neiend, neistart);
  while ss < 0
     neiend     = neix;
     [neix, ss] = findneix(E, neiend, neistart);
  end
% 
% -------------------------------------------------------------------------
% Form compressed covariance matrix based on the neix largest eigenvalues.
%
  [Qc, Qs, D] = Qgen(neix, CC, SS, CS);
%  
% Calculate the nei largest eigenvalues of the approximation to covariance
% matrix.
%
  Eapp = eigs(@(x)CVappfun(x,sparse(Qc),sparse(Qs),sparse(D)), ...
     2*n, nei, 'lm', opts);
% 
% -------------------------------------------------------------------------
% Build array RESULT to store results in a single data structure.
%
  DC     = mean(dc);
  Param  = [length(f); DC; neix; f(1)];
  nparam = length(Param);
  Q      = [Qc; Qs];
  RESULT = [nparam; Param; real(zref); imag(zref); ...
            D(:,1); D(:,3); D(:,2); Q(:)];
  name   = [name, '.mat'];
%
% Save to MAT file.
%
  [fname, pname] = uiputfile({'*.mat'}, 'Select a RESULT file', name);
  save([pname, fname], 'RESULT')
%
% -------------------------------------------------------------------------
% Compare full and compressed covariance matrices.
%
  CV              = [CC,  CS';  CS,  SS];
  [CCx, SSx, CSx] = CVgen(Qc, Qs, D);
  CVx             = [CCx, CSx'; CSx, SSx];
  [~, p]          = chol(CVx);
  SR              = 100*(2*n*neix + 3*n)/(2*n)^2;
  Err             = 100*sqrt(sum(sum((CV - CVx).^2))/sum(sum(CV.^2)));
%
% Numerical output.
%
  format compact
  disp('Storage ratio/%:')
  disp(SR)
  disp('Relative error/%:')
  disp(Err)
  disp('Number of eigenvalues/1:')
  disp(neix)
  if p == 0
     disp('Covariance matrix is positive-definite')
  else
     disp('Covariance matrix is NOT positive-definite')
  end
%
% Graphical output.
%
  figure, semilogy((1:nei), E, 'o', (1:nei), Eapp, 'x'), hold on
  plot([neix, neix], ylim, 'k--')
  title(['Eigenvalues of covariance matrix CV: neix = ', int2str(neix)])
  xlabel('Index of eigenvalue')
  ylabel('Eigenvalue')
  legend('Full matrix', 'Compact matrix', 0)
%
% Graphical output (optional).
%
  if icontrol(2) == 1
     figure, surf(CV),     shading interp, view(115,30)
     title('Full covariance matrix CV')
     figure, surf(CVx),    shading interp, view(115,30)
     title('Compressed covariance matrix CVx')
     figure, surf(CV-CVx), shading interp, view(115,30)
     title('Differences CV-CVx')
  end

function [neix, ss] = findneix(E, neiend, neistart)
  LE   = log10(E(1:neiend));
  k1   = neistart;
  k2   = round(neiend/2);
  flag = 0;
  while ~flag
     A1   = [ones(k2-k1+1,1),  (k1:k2)'];
     b1   = A1\LE(k1:k2);
     A2   = [ones(neiend-k2+1,1), (k2:neiend)'];
     b2   = A2\LE(k2:neiend);
     neix = round((b1(1) - b2(1))/(b2(2) - b1(2)));
     figure(1)
     plot((1:neiend), LE, 'o'), hold on
     plot([k2, k2],     [min(LE), max(LE)],                'k-')
     plot([neix, neix], [min(LE), max(LE)],                'k--')
     plot([1, neiend],  [b1(1)+b1(2), b1(1)+b1(2)*neiend], 'b-')
     plot([1, neiend],  [b2(1)+b2(2), b2(1)+b2(2)*neiend], 'r-')
     ylim([min(LE), max(LE)])
     pause, hold off
     if neix == k2
        flag = 1;
        ss   = sign(b2(2) - b1(2));
     end
     if neix < k1+2 || neix > neiend-2
        flag = 1;
        neix = round(neiend/2);
        ss   = -1;
     end
     k2 = neix;
  end
  close(1)
%
% -------------------------------------------------------------------------
% End.
%
% Copyright NPL. No warranty or guarantee applies to this software, and 
% therefore any users should satisfy themselves that it meets their 
% requirements.
% -------------------------------------------------------------------------