function RESULT = CreateCompactCV_eig_compact(icontrol)
% -------------------------------------------------------------------------
% CreateCompactCV_eig_compact.m
%
% Author         David Humphreys and Peter Harris, NPL
% Created        24-07-2013   From CreateCompact_eig.m to avoid forming 
%                             full covariance matrix.
% Last amended   26-03-2014   Added DC value.
%                31-03-2014   Added test of postive-definiteness.
%                13-04-2014   Alignment made optional.
% Released       30-06-2014
% -------------------------------------------------------------------------
% Create compact representation of covariance matrix for harmonic results
% based on eigendecomposition without storing the full covariance matrix.
%
% See: Guidance for the use of compact covariance uncertainty software,
%      David Humphreys and Peter Harris, NPL REPORT TQE 10, 2014
% -------------------------------------------------------------------------
% Input:
%    icontrol    2 x 1        Control of alignment and comparison 
%                             (optional 0): default = [1, 0].
%
% Output:
%    RESULT                   Array storing compact representation of
%                             covariance matrix.
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Set defaults for optional input parameter.
%
  if nargin < 1
     icontrol = [1, 0];
  end
%  
% -------------------------------------------------------------------------
% Prompt user for path and name of the file containing the measured
% waveforms (.csv files only).
%
  [fname, pname] = uigetfile({'*.csv'}, 'Select a DATA file');
  [~, name]      = fileparts(fname);
%
% Create temporary subdirectory to contain files storing individual
% waveforms.
%
  disp('Loading and saving individual waveforms ...')
  mkdir(pname, 'tempdata');
%
% Load frequencies and determine the number n frequencies (excluding DC 
% component).
%
  GHz = 10^9;
  col = ExcelCol(1);
  f   = xlsread([pname, fname], [col{:},':',col{:}]);
  n   = length(f) - 1;
  f   = f(2:n+1)/GHz;
%
% Determine the number m of waveforms, and store the individual waveforms
% in temporary MAT-files.
%
  m = 0;
  while 1
     col  = ExcelCol([2*m+2, 2*m+3]);
     col1 = col(1);
     col2 = col(2);
     z    = xlsread([pname, fname], [col1{:},'2:',col2{:},int2str(n+1)]);
     if isempty(z)
        break
     end
     m = m + 1;
     save([pname, 'tempdata\', int2str(m)], 'z');
  end
%
% Load DC values.
%
  col  = ExcelCol([2, 2*m+1]);
  col1 = col(1);
  col2 = col(2);
  dc   = xlsread([pname, fname], [col1{:},'1:',col2{:},'1']);
  dc   = dc(1:2:end); 
%
% -------------------------------------------------------------------------
% Pre-processing: alignment of harmonic results with mean zref, which
% overwrites the individual waveforms stored in temporary MAT-files.
%
  if icontrol(1) == 1
     disp('Alignment of individual waveforms ...')
     alignment_compact(pname, n, m);
  end
%
% -------------------------------------------------------------------------
% Calculate average of the waveforms, and the rows and diagonals of the 
% full covariance matrix by applying an updating procedure, and store the 
% information in temporary MAT-files.
%
  disp('Saving information about full covariance matrix ...')
  mkdir(pname, 'tempcv');
  kcc = 1:n;
  kss = n+1:2*n;
  for i = 1:2*n
     for j = 1:m
        load([pname, 'tempdata\', int2str(j)]);
        z = [z(:,1); z(:,2)];
        if j == 1
           if i == 1
              VCC0 = zeros(n, 1);
              VCS0 = zeros(n, 1);
              VSS0 = zeros(n, 1);
           end
           Vzi0  = zeros(1,2*n);
           zref0 = z;
        else
           zref = zref0 + (z - zref0)/j;
           Vzi  = ((j-2)*Vzi0 ...
              + (j-1)*(zref(i)-zref0(i))*(zref-zref0)' ...
              + (z(i)-zref(i))*(z-zref)')/(j-1);
           if i == 1
              VCC  = ((j-2)*VCC0 ...
                 + (j-1)*(zref(kcc)-zref0(kcc)).*(zref(kcc)-zref0(kcc)) ...
                 + (z(kcc)-zref(kcc)).*(z(kcc)-zref(kcc)))/(j-1);
              VCS  = ((j-2)*VCS0 ...
                 + (j-1)*(zref(kcc)-zref0(kcc)).*(zref(kss)-zref0(kss)) ...
                 + (z(kcc)-zref(kcc)).*(z(kss)-zref(kss)))/(j-1);
              VSS  = ((j-2)*VSS0 ...
                 + (j-1)*(zref(kss)-zref0(kss)).*(zref(kss)-zref0(kss)) ...
                 + (z(kss)-zref(kss)).*(z(kss)-zref(kss)))/(j-1);
              VCC0 = VCC;
              VCS0 = VCS;
              VSS0 = VSS;
           end
           Vzi0  = Vzi;
           zref0 = zref;
        end
     end
     Vzi0 = Vzi0/m;
     save([pname, 'tempcv\', int2str(i)], 'Vzi0');
  end
  VCC0 = VCC0/m;
  VCS0 = VCS0/m;
  VSS0 = VSS0/m;
%
% -------------------------------------------------------------------------
% Calculate the nei largest eigenvalues of the full covariance matrix.
%
  disp('Determining number of components in compact representation ...')
  nei = min([2*n-1, m-1]);
  opts.issym = 1;
  E = eigs(@(x)CVfun_compact(x,pname,n), 2*n, nei, 'lm', opts);
%
% Decide how many eigenvalues to use to form approximation to covariance
% matrix.
%
  neistart   = 2;
  neiend     = nei;
  [neix, ss] = findneix(E, neiend, neistart);
  while ss < 0
     neiend     = neix;
     [neix, ss] = findneix(E, neiend, neistart);
  end
%
% Graphical output.
%
  figure, semilogy(E, 'o'), hold on
  plot([neix, neix], ylim, 'k--')
  title(['Eigenvalues of covariance matrix CV: neix = ', int2str(neix)])
  xlabel('Index of eigenvalue')
  ylabel('Eigenvalue')
% 
% -------------------------------------------------------------------------
% Form compressed covariance matrix based on the neix largest eigenvalues.
%
  disp('Forming compact representation ...')
  [Qc, Qs, D] = Qgen_compact(neix, pname, n, VCC0, VCS0, VSS0);
%
% -------------------------------------------------------------------------
% Build array RESULT to store results in a single data structure.
%
  disp('Saving compact representation ...')
  DC     = mean(dc);
  Param  = [n; DC; neix; f(1)];
  nparam = length(Param);
  Q      = [Qc; Qs];
  RESULT = [nparam; Param; zref0; D(:,1); D(:,3); D(:,2); Q(:)];
  name   = [name, '.mat'];
%
% Save to MAT file.
%
  [fname_out, pname_out] = ...
     uiputfile({'*.mat'}, 'Select a RESULT file', name);
  save([pname_out, fname_out], 'RESULT')
%
% Remove temporary MAT-files.
%
  disp('Removing temporary files ...')
  delete([pname, 'tempdata\*.mat'])
  rmdir([pname, 'tempdata'])
  delete([pname, 'tempcv\*.mat'])
  rmdir([pname, 'tempcv'])
%
% -------------------------------------------------------------------------
% If selected, compare full and compressed covariance matrices.
%
  if icontrol(2) == 1
     disp('Comparison with full covariance matrix ...')
%
%    Form full and compressed covariance matrices.
%
     M               = xlsread([pname, fname]);
     z               = M(2:n+1,2:2:1+2*m) + 1i*M(2:n+1,3:2:1+2*m);
     [z, zref]       = alignment(z);
     [CC, SS, CS]    = fullcovariance(z, zref);
     CV              = [CC,  CS';  CS,  SS];
     [CCx, SSx, CSx] = CVgen(Qc, Qs, D);
     CVx             = [CCx, CSx'; CSx, SSx];
     [~, p]          = chol(CVx);
     SR              = 100*(2*n*neix + 3*n)/(2*n)^2;
     Err             = 100*sqrt(sum(sum((CV - CVx).^2))/sum(sum(CV.^2)));
%
%    Numerical output.
%
     format compact
     disp('Storage ratio/%:')
     disp(SR)
     disp('Relative error/%:')
     disp(Err)
     disp('Number of eigenvalues/1:')
     disp(neix)
     if p == 0
        disp('Covariance matrix is positive-definite')
     else
        disp('Covariance matrix is NOT positive-definite')
     end
%
%    Graphical output.
%
     figure, surf(CV),     shading interp, view(115,30)
     title('Full covariance matrix CV')
     figure, surf(CVx),    shading interp, view(115,30)
     title('Compressed covariance matrix CVx')
     figure, surf(CV-CVx), shading interp, view(115,30)
     title('Differences CV-CVx')
  end

function [neix, ss] = findneix(E, neiend, neistart)
  LE   = log10(E(1:neiend));
  k1   = neistart;
  k2   = round(neiend/2);
  flag = 0;
  while ~flag
     A1   = [ones(k2-k1+1,1),  (k1:k2)'];
     b1   = A1\LE(k1:k2);
     A2   = [ones(neiend-k2+1,1), (k2:neiend)'];
     b2   = A2\LE(k2:neiend);
     neix = round((b1(1) - b2(1))/(b2(2) - b1(2)));
     figure(1)
     plot((1:neiend), LE, 'o'), hold on
     plot([k2, k2],     [min(LE), max(LE)],                'k-')
     plot([neix, neix], [min(LE), max(LE)],                'k--')
     plot([1, neiend],  [b1(1)+b1(2), b1(1)+b1(2)*neiend], 'b-')
     plot([1, neiend],  [b2(1)+b2(2), b2(1)+b2(2)*neiend], 'r-')
     ylim([min(LE), max(LE)])
     pause, hold off
     if neix == k2
        flag = 1;
        ss   = sign(b2(2) - b1(2));
     end
     if neix < k1+2 || neix > neiend-2
        flag = 1;
        neix = round(neiend/2);
        ss   = -1;
     end
     k2 = neix;
  end
  close(1)
%
% -------------------------------------------------------------------------
% End.
%
% Copyright NPL. No warranty or guarantee applies to this software, and 
% therefore any users should satisfy themselves that it meets their 
% requirements.
% -------------------------------------------------------------------------