function RESULT = FormConstantCompactCV_eig(DC, f0, zref, uzref, n)
% -------------------------------------------------------------------------
% FormConstantCompactCV_eig.m
%
% Author         David Humphreys and Peter Harris, NPL
% Created        07-02-2013
% Last amended   07-02-2013
% Released       08-02-2013
% Last amended   15-07-2013   Incorporated path name.
%                13-04-2014   Added DC and input/output parameters.
% Released       30-06-2014
% -------------------------------------------------------------------------
% Form compact representation of covariance matrix for user-supplied
% waveform transform.
%
% See: Guidance for the use of compact covariance uncertainty software,
%      David Humphreys and Peter Harris, NPL REPORT TQE 10, 2014
% -------------------------------------------------------------------------
% Input:
%    DC          1 x 1        DC value.
%    f0          1 x 1        Frequency spacing (in GHz).
%    zref                     Complex-valued waveform transform:
%                                Rzref    = real(zref), 
%                                Izref    = imag(zref).
%    uzref                    Complex-valued standard uncertainties:
%                                u(Rzref) = real(uzref),
%                                u(Izref) = imag(uzref).
%    n           1 x 1        Number of frequencies (excluding DC): used
%                                only if length(zref) = 1.
%
% Output:
%    RESULT                   Array storing compact representation of
%                             covariance matrix.
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Form real and imaginary parts of waveform transform and the associated
% covariance matrix in compact form, according to the information provided
% by the user.
%
  if length(zref) > 1
%
%   The waveform transform and associated standard uncertainties are
%   provided.
%
    n     = length(zref);
    neix  = 1;
    Rzref = real(zref);
    Izref = imag(zref);
    D     = [real(uzref).^2, zeros(n,1), imag(uzref).^2];
    Qc    = zeros(n,1);
    Qs    = zeros(n,1);
%
  else
%
%   A constant waveform transform with associated standard uncertainty is
%   provided.
%
    neix  = 1;
    Rzref = ones(n,1)*real(zref);
    Izref = ones(n,1)*imag(zref);
    D     = [ones(n,1)*real(uzref)^2, zeros(n,1), ones(n,1)*imag(uzref)^2];
    Qc    = zeros(n,1);
    Qs    = zeros(n,1);
%    
  end
%
% -------------------------------------------------------------------------
% Build array RESULT to store results in a single data structure.
%
  Param  = [n; DC; neix; f0];
  nparam = length(Param);
  Q      = [Qc; Qs];
  RESULT = [nparam; Param; Rzref; Izref; D(:,1); D(:,3); D(:,2); Q(:)];
  name   = ['CONST ', num2str(f0), ' GHz ', int2str(n), '.mat'];
%
% Save to MAT file.
%
  [fname, pname] = uiputfile({'*.mat'}, 'Select a RESULT file', name);
  save([pname, fname], 'RESULT')
%
% -------------------------------------------------------------------------
% End.
%
% Copyright NPL. No warranty or guarantee applies to this software, and 
% therefore any users should satisfy themselves that it meets their 
% requirements.
% -------------------------------------------------------------------------