function [Aref, uAref, Pref, uPref] = ...
  FormFreqRespFromCompactCV_eig(RESULT)
% -------------------------------------------------------------------------
% FormFreqRespFromCompactCV_eig.m
%
% Author         David Humphreys and Peter Harris, NPL
% Created        18-06-2012
% Last amended   25-01-2013   Added LoadCompactCV_eig.m.
%                30-01-2013   Renamed. 
%                             Added treatment of zero magnitude.
%                             Added treatment of phase.
%                05-02-2013   Added input parameter.
%                06-02-2013   Updated.
% Released       08-02-2013
% Last amended   15-07-2013   Incorporated path name.
%                13-04-2014   Added input/output parameters.
% Released       30-06-2014
% -------------------------------------------------------------------------
% Form frequency response and associated standard uncertainties from 
% waveform transform and compact representatation of covariance matrix
% by applying the law of propagation of uncertainty.
%
% See: Guidance for the use of compact covariance uncertainty software,
%      David Humphreys and Peter Harris, NPL REPORT TQE 10, 2014
% -------------------------------------------------------------------------
% Input:
%    RESULT                   Array storing compact representation of
%                             covariance matrix (optional).
%
% Output:
%    Aref        n x 1        
%    uAref       n x 1        Amplitude values and associated standard 
%                             uncertainties.
%    Pref        n x 1
%    uPref       n x 1        Phase values and associated standard
%                             uncertainties.
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% If required, select a file containing an array RESULT, and load array.
%
  if nargin < 1
     [fname, pname] = uigetfile('*.mat', 'Select a RESULT file');
     load([pname, fname])
  end
%
% -------------------------------------------------------------------------
% Extract information about compressed covariance matrix from array RESULT.
%
  [n, ~, ~, f0, Rzref, Izref, D, Qc, Qs] = LoadCompactCV_eig(RESULT);
%
% -------------------------------------------------------------------------
% Form amplitudes and phases, and associated uncertainties.
%
  Aref  = NaN*ones(n,1);
  uAref = NaN*ones(n,1);
  Pref  = NaN*ones(n,1);
  uPref = NaN*ones(n,1);
  for k = 1:n
%
%    Estimates.
%
     Aref(k) = sqrt(Rzref(k)^2 + Izref(k)^2);
     Pref(k) = atan2(Izref(k), Rzref(k));
%
%    Associated standard uncertainties.
%
     if Aref(k) > 0
%
%       Sensitivity coefficients for amplitude.
%
        Ac(1,1)  =  Rzref(k)/Aref(k);
        Ac(1,2)  =  Izref(k)/Aref(k);
%
%       Sensitivity coefficients for phase.
%
        Pc(1,1)  = -Izref(k)/Aref(k);
        Pc(1,2)  =  Rzref(k)/Aref(k);
%
%       Form covariance matrix for components of kth harmonic.
%
        Vk(1,1) = D(k,1) + Qc(k,:)*Qc(k,:)';
        Vk(2,1) = D(k,2) + Qs(k,:)*Qc(k,:)';
        Vk(1,2) = Vk(2,1);
        Vk(2,2) = D(k,3) + Qs(k,:)*Qs(k,:)';
%
%       Application of law of propagation of uncertainty.
%
        uAref(k) = sqrt(Ac*Vk*Ac');
        uPref(k) = sqrt(Pc*Vk*Pc');
%
     end
%
  end
%
% -------------------------------------------------------------------------
% Plot amplitude response and associated standard uncertainties.
%
  figure, plot(f0*(1:n)',  Aref, 'x')
  xlabel('Frequency/GHz')
  ylabel('Amplitude response')
  figure, plot(f0*(1:n)', uAref, 'o')
  xlabel('Frequency/GHz')
  ylabel('Amplitude standard uncertainties')
%
% -------------------------------------------------------------------------
% Plot phase response and associated standard uncertainties.
%
  figure, plot(f0*(1:n)',  Pref, 'x')
  xlabel('Frequency/GHz')
  ylabel('Phase response')
  figure, plot(f0*(1:n)', uPref, 'o')
  xlabel('Frequency/GHz')
  ylabel('Phase standard uncertainties')
%
% -------------------------------------------------------------------------
% End.
%
% Copyright NPL. No warranty or guarantee applies to this software, and 
% therefore any users should satisfy themselves that it meets their 
% requirements.
% -------------------------------------------------------------------------