function [Aref, uAref, Pref, uPref] = ...
  FormFreqRespFromCompactCV_eig_MC(RESULT)
% -------------------------------------------------------------------------
% FormFreqRespFromCompactCV_eig_MC.m
%
% Author         David Humphreys and Peter Harris, NPL
% Created        06-05-2013   From FormFreqRespFromCompactCV_eig.m to
%                             use Monte Carlo determination of
%                             estimates and associated standard 
%                             uncertainties.
% Last amended   15-07-2013   Incorporated path name.
%                25-07-2013   Updated.
%                13-04-2014   Added input/output parameters.
% Released       30-06-2014
% -------------------------------------------------------------------------
% Form frequency response and associated standard uncertainties from 
% waveform transform and compact representatation of covariance matrix.
% by applying a Monte Carlo method.
%
% See: Guidance for the use of compact covariance uncertainty software,
%      David Humphreys and Peter Harris, NPL REPORT TQE 10, 2014
% -------------------------------------------------------------------------
% Input:
%    RESULT                   Array storing compact representation of
%                             covariance matrix (optional).
%
% Output:
%    Aref        n x 1        
%    uAref       n x 1        Amplitude values and associated standard 
%                             uncertainties.
%    Pref        n x 1
%    uPref       n x 1        Phase values and associated standard
%                             uncertainties.
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% If required, select a file containing an array RESULT, and load array.
%
  if nargin < 1
     [fname, pname] = uigetfile('*.mat', 'Select a RESULT file');
     load([pname, fname])
  end
%
% -------------------------------------------------------------------------
% Extract information about compressed covariance matrix from array RESULT.
%
  [n, ~, ~, f0, Rzref, Izref, D, Qc, Qs] = LoadCompactCV_eig(RESULT);
%
% -------------------------------------------------------------------------
% Form amplitudes and phases, and associated uncertainties.
%
  Aref    = NaN*ones(n,1);
  uAref   = NaN*ones(n,1);
  Pref    = NaN*ones(n,1);
  uPref   = NaN*ones(n,1);
  MCtrial = 1000;
  zstdN   = randn(2,MCtrial);
  for k = 1:n
%
%    Form covariance matrix for components of kth harmonic.
%
     Vk(1,1) = D(k,1) + Qc(k,:)*Qc(k,:)';
     Vk(2,1) = D(k,2) + Qs(k,:)*Qc(k,:)';
     Vk(1,2) = Vk(2,1);
     Vk(2,2) = D(k,3) + Qs(k,:)*Qs(k,:)';
%
%    Samples of real and imaginary parts.
%
     Lk = chol(Vk, 'lower');
     zk = [Rzref(k); Izref(k)]*ones(1,MCtrial) + Lk*zstdN;
%
%    Apply rotation so that estimate of phase is approximatley zero.
%
     Pref0 = atan2(Izref(k), Rzref(k));
     R     = [cos(Pref0), sin(Pref0); -sin(Pref0), cos(Pref0)];
     Rzk   = R*zk;
%
%    Values of amplitude and phase (rotated).
%
     Ak = sqrt(Rzk(1,:).^2 + Rzk(2,:).^2);
     Pk = atan2(Rzk(2,:), Rzk(1,:));
%
%    Estimates.
%
     Aref(k) = mean(Ak);
     Pref(k) = mean(Pk) + Pref0;
%
%    Associated standard uncertainties.
%
     uAref(k) = std(Ak);
     uPref(k) = std(Pk);
%
  end
%
% -------------------------------------------------------------------------
% Plot amplitude response and associated standard uncertainties.
%
  figure, plot(f0*(1:n)',  Aref, 'x')
  xlabel('Frequency/GHz')
  ylabel('Amplitude response')
  figure, plot(f0*(1:n)', uAref, 'o')
  xlabel('Frequency/GHz')
  ylabel('Amplitude standard uncertainties')
%
% -------------------------------------------------------------------------
% Plot phase response and associated standard uncertainties.
%
  figure, plot(f0*(1:n)',  Pref, 'x')
  xlabel('Frequency/GHz')
  ylabel('Phase response')
  figure, plot(f0*(1:n)', uPref, 'o')
  xlabel('Frequency/GHz')
  ylabel('Phase standard uncertainties')
%
% -------------------------------------------------------------------------
% End.
%
% Copyright NPL. No warranty or guarantee applies to this software, and 
% therefore any users should satisfy themselves that it meets their 
% requirements.
% -------------------------------------------------------------------------