function RESULT = FormMultiplyFromCompactCV_eig
% -------------------------------------------------------------------------
% FormMultiplyFromCompactCV_eig.m
%
% Author         David Humphreys and Peter Harris, NPL
% Created        31-01-2013
% Last amended   06-02-2013
% Released       08-02-2013
% Last amended   15-07-2013   Incorporated path names.
%                02-05-2014   Added input/output parameters.
% Released       30-06-2014
% -------------------------------------------------------------------------
% Form product of waveform transforms and associated covariance matrix
% in compact form.
%
% See: Guidance for the use of compact covariance uncertainty software,
%      David Humphreys and Peter Harris, NPL REPORT TQE 10, 2014
% -------------------------------------------------------------------------
% Output:
%    RESULT                   Array storing compact representation of
%                             covariance matrix for product of waveform
%                             transforms.
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% For the first waveform, select a file containing an array RESULT, 
% and load array.
%
  [fname1, pname1] = uigetfile('*.mat', 'Select a RESULT file');
  load([pname1, fname1])
  [~, name1] = fileparts(fname1);
%
% Extract information about compressed covariance matrix from array RESULT.
%
  [n1, DC1, neix1, f01, Rzref1, Izref1, D1, Qc1, Qs1] = ...
     LoadCompactCV_eig(RESULT); %#ok<NODEF>
%
% -------------------------------------------------------------------------
% For the second waveform, select a file containing an array RESULT, 
% and load array.
%
  [fname2, pname2] = uigetfile('*.mat', 'Select a RESULT file');
  load([pname2, fname2])
  [~, name2] = fileparts(fname2);
%
% Extract information about compressed covariance matrix from array RESULT.
%
  [n2, DC2, neix2, f02, Rzref2, Izref2, D2, Qc2, Qs2] = ...
     LoadCompactCV_eig(RESULT);
%
% -------------------------------------------------------------------------
% Check compatability of waveforms.
%
  if (n1 ~= n2) || (f01 ~= f02)
     error('Waveform transforms incompatible for MULTIPLY.')
  end
%
% -------------------------------------------------------------------------
% Form product of waveform transforms and the associated covariance matrix
% in compact form.
%
  n     = n1;
  DC    = DC1*DC2;
  neix  = neix1 + neix2;
  f0    = f01;
  Rzref = zeros(n,1);
  Izref = zeros(n,1);
  Qca   = zeros(n,neix1);
  Qsa   = zeros(n,neix1);
  Da    = zeros(n,3);
  Qcb   = zeros(n,neix2);
  Qsb   = zeros(n,neix2);
  Db    = zeros(n,3);
  for k = 1:n
%
%    Estimate.
%
     Rzref(k) = Rzref1(k)*Rzref2(k) - Izref1(k)*Izref2(k);
     Izref(k) = Rzref1(k)*Izref2(k) + Izref1(k)*Rzref2(k);
%
%    Associated covariance matrix (represented in compact form) ...
%
%    ... contribution from first waveform:
%
     cc2      = zeros(1,2);
     cc2(1)   = Rzref2(k);
     cc2(2)   = -Izref2(k);
     Qca(k,:) = cc2*[Qc1(k,:); Qs1(k,:)];
     cs2      = zeros(1,2);
     cs2(1)   = Izref2(k);
     cs2(2)   = Rzref2(k);
     Qsa(k,:) = cs2*[Qc1(k,:); Qs1(k,:)];
     C2       = [cc2; cs2];
     VD1      = [D1(k,1), D1(k,2); D1(k,2), D1(k,3)];
     VDa      = C2*VD1*C2';
     Da(k,1)  = VDa(1,1);
     Da(k,2)  = VDa(1,2);
     Da(k,3)  = VDa(2,2);
%
%    ... contribution from second waveform:
%
     cc1      = zeros(1,2);
     cc1(1)   = Rzref1(k);
     cc1(2)   = -Izref1(k);
     Qcb(k,:) = cc1*[Qc2(k,:); Qs2(k,:)];
     cs1      = zeros(1,2);
     cs1(1)   = Izref1(k);
     cs1(2)   = Rzref1(k);
     Qsb(k,:) = cs1*[Qc2(k,:); Qs2(k,:)];
     C1       = [cc1; cs1];
     VD2      = [D2(k,1), D2(k,2); D2(k,2), D2(k,3)];
     VDb      = C1*VD2*C1';
     Db(k,1)  = VDb(1,1);
     Db(k,2)  = VDb(1,2);
     Db(k,3)  = VDb(2,2);
%
  end
%
% -------------------------------------------------------------------------
% Build array RESULT to store results in a single data structure.
%
  Param  = [n; DC; neix; f0];
  nparam = length(Param);
  D      = Da + Db;
  Q      = [[Qca, Qcb]; [Qsa, Qsb]];
  RESULT = [nparam; Param; Rzref; Izref; D(:,1); D(:,3); D(:,2); Q(:)];
  name   = [name1, ' MULT ', name2, '.mat'];
%
% Save to MAT file.
%
  [fname, pname] = uiputfile({'*.mat'}, 'Select a RESULT file', name);
  save([pname, fname], 'RESULT')
%
% -------------------------------------------------------------------------
% End.
%
% Copyright NPL. No warranty or guarantee applies to this software, and 
% therefore any users should satisfy themselves that it meets their 
% requirements.
% -------------------------------------------------------------------------