function [t, tref, utref] = FormTimeSeriesFromCompactCV_eig(n0, s0, RESULT)
% -------------------------------------------------------------------------
% FormTimeSeriesFromCompactCV_eig.m
%
% Author         David Humphreys and Peter Harris, NPL
% Created        30-01-2013
% Last amended   06-02-2013
% Released       08-02-2013
%                15-07-2013   Incorporated path name.
%                01-04-2014   Added DC; added input/output parameters.
%                13-04-2014   Added interpolation.
% Released       30-06-2014
% -------------------------------------------------------------------------
% Form time series and associated standard uncertainties from 
% compact representatation of covariance matrix.
%
% See: Guidance for the use of compact covariance uncertainty software,
%      David Humphreys and Peter Harris, NPL REPORT TQE 10, 2014
% -------------------------------------------------------------------------
% Input:
%    n0          1 x 1        Number of time values (can be zero).
%    s0          1 x 1        Shift in number of time points (can be zero).
%    RESULT                   Array storing compact representation of
%                             covariance matrix (optional.
%
% Output:
%    t          nt x 1        Time values.
%    tref       nt x 1        
%    utref      nt x 1        Waveform values and associated standard 
%                             uncertainties.
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% If required, select a file containing an array RESULT, and load array.
%
  if nargin < 3
     [fname, pname] = uigetfile('*.mat', 'Select a RESULT file');
     load([pname, fname])
  end
%
% -------------------------------------------------------------------------
% Extract information about compressed covariance matrix from array RESULT.
%
  [n, DC, ~, f0, Rzref, Izref, D, Qc, Qs] = LoadCompactCV_eig(RESULT);
%
% Set signal length L and original sampling interval dt.
%
  GHz = 10^9;
  L   = 1/(f0*GHz);
  dt  = 1/(2*n*f0*GHz);
%
% Set time points: if number nt of points is not set by the user, nt 
% depends on the number of harmonics and is even or odd according to 
% whether Izref(n) is zero or not.
%    
  if n0 == 0
     if Izref(n) == 0
        nt = 2*n;
     else
        nt = 2*n + 1;
     end
     t = (0:nt-1)'*dt;
  else
     nt = n0;
     if Izref(n) == 0
        t = linspace(0, L, nt+1)';
        t = t(1:nt);
     else
        t = linspace(0, L+dt, nt+1)';
        t = t(1:nt);
     end 
  end
%
% -------------------------------------------------------------------------
% Form signal estimates and associated standard uncertainties.
%
  T     = 2*pi*(-s0:nt-1-s0)/nt;
  tref  = DC*ones(nt,1);
  utref = zeros(nt,1);
  for j = 1:nt
%
%    Sensitivity coefficients.
%
     C = cos(T(j)*(1:n));
     S = sin(T(j)*(1:n));
%
%    Estimates.
%
     tref(j) = tref(j) + (C*Rzref + S*Izref);
%
%    Standard uncertinties.
%
     v1       = C*Qc + S*Qs;
     v1       = v1*v1';
     v2       = [(C.*D(:,1)'+S.*D(:,2)'), (C.*D(:,2)'+S.*D(:,3)')];
     v2       = v2*[C'; S'];
     utref(j) = sqrt(v1 + v2);
  end
%  
% -------------------------------------------------------------------------
% Plot waveform and associated standard uncertainties.
%
  figure, plot(t,  tref, 'x-')
  xlabel('Time/s')
  ylabel('Waveform')
  figure, plot(t, utref, 'o-')
  xlabel('Time/s')
  ylabel('Waveform standard uncertainties')
%
% -------------------------------------------------------------------------
% End.
%
% Copyright NPL. No warranty or guarantee applies to this software, and 
% therefore any users should satisfy themselves that it meets their 
% requirements.
% -------------------------------------------------------------------------