function [n, DC, neix, f0, Rzref, Izref, D, Qc, Qs] = ...
    LoadCompactCV_eig(RESULT)
% -------------------------------------------------------------------------
% LoadCompactCV_eig.m
%
% Author         David Humphreys and Peter Harris, NPL
% Created        25-01-2013
% Last amended   30-01-2013   Change to output parameters.
% Released       08-02-2013
% Last amended   31-03-2014   Added input/output parameters.
% Released       30-06-2014
% -------------------------------------------------------------------------
% Unpack information in compact representation of covariance matrix for 
% harmonic results based on eigendecomposition.
%
% See: Guidance for the use of compact covariance uncertainty software,
%      David Humphreys and Peter Harris, NPL REPORT TQE 10, 2014
% -------------------------------------------------------------------------
% Input:
%    RESULT                   Array storing compact representation of
%                             covariance matrix.
%
% Output:
%    n           1 x 1        Number of frequencies (excluding DC).
%    DC          1 x 1        DC value.
%    neix        1 x 1        Number of eigenvalues used in approximation
%                             of covariance matrix.
%    f0          1 x 1        Frequency spacing (in GHz).
%    Rzref       n x 1        Real part of aligned and averaged harmomic 
%                             results.
%    Izref       n x 1        Imaginary part of aligned and averaged
%                             harmonic results.
%    D          3n x 1        Array storing corrections to diagonal parts
%                             in compact representation of covariance 
%                             matrix.
%    Qc,         n x neix
%    Qs          n x neix     Arrays storing principal components in
%                             compact representation of covariance matrix.
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Extract basic information from array RESULT.
%
  np    = RESULT(1);
  n     = RESULT(2);
  DC    = RESULT(3);
  neix  = RESULT(4);
  f0    = RESULT(5);
%
% Extract remaining information after checking the length of the array.
%
  if length(RESULT) == 1+np+5*n+2*n*neix
     Rzref = RESULT(1+np+1:1+np+n);
     Izref = RESULT(1+np+n+1:1+np+2*n);
     D1    = RESULT(1+np+2*n+1:1+np+4*n);
     D2    = RESULT(1+np+4*n+1:1+np+5*n);
     Q     = RESULT(1+np+5*n+1:1+np+5*n+2*n*neix);
  else
     error('Array RESULT of incorrect length')
  end
  D  = [D1(1:n), D2, D1(1+n:2*n)];
  Q  = reshape(Q, 2*n, neix);
  Qc = Q(1:n,:);
  Qs = Q(1+n:2*n,:);
%
% -------------------------------------------------------------------------
% End.
%
% Copyright NPL. No warranty or guarantee applies to this software, and 
% therefore any users should satisfy themselves that it meets their 
% requirements.
% -------------------------------------------------------------------------