function [Qc, Qs, D] = Qgen(nei, CC, SS, CS)
% -------------------------------------------------------------------------
% Qgen.m
%
% Author         David Humphreys and Peter Harris, NPL
% Created        07-06-2012
% Last amended   07-06-2012
% Released       08-02-2013
% Last amended   26-03-2014   Added input/output parameters.
% Released       30-06-2014
% -------------------------------------------------------------------------
% Create compact representation of covariance matrix defined by a 
% specific number of eigenvalues (principal components).
%
% See: Guidance for the use of compact covariance uncertainty software,
%      David Humphreys and Peter Harris, NPL REPORT TQE 10, 2014
% -------------------------------------------------------------------------
% Input:
%    nei         1 x 1        Number of eigenvalues.
%    CC,
%    SS,
%    CS          n x n        Submatrices defining full covariance matrix:
%                                V = [CC, CS'; CS, SS]
%
% Output:
%    Qc,
%    Qs          n x nei
%    D           n x 3        Submatrices defining compact representation.
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Number n of frequencies (excluding DC).
%
  n = size(CC,1);
%
% Evaluate the nei largest eigenvalues and corresponding eigenvectors.
%
  opts.issym = 1;
  [Q, E] = eigs(@(x)CVfun(x,sparse(CC),sparse(SS),sparse(CS)), ...
              2*n, nei, 'lm', opts);
%
% Form output: (i) eignevectors scaled by the square roots of the 
% eigenvalues stored as the sub-matrices Qc and Qs corresponding to the
% Re and Im parts; (ii) differences between the "diagonals" of the full
% and approximate covariance matrices for the Re/Re, Im/Im and Im/Re parts. 
%
  Qc     = Q(1:n,:)*sqrt(E);
  Qs     = Q(1+n:2*n,:)*sqrt(E);
  D(:,1) = diag(CC) - diag(Qc*Qc');
  D(:,2) = diag(CS) - diag(Qs*Qc');
  D(:,3) = diag(SS) - diag(Qs*Qs');
%
% -------------------------------------------------------------------------
% End.
%
% Copyright NPL. No warranty or guarantee applies to this software, and 
% therefore any users should satisfy themselves that it meets their 
% requirements.
% -------------------------------------------------------------------------