function [Qc, Qs, D] = Qgen_compact(nei, pname, n, VCC0, VCS0, VSS0)
% -------------------------------------------------------------------------
% Qgen_compact.m
%
% Author         David Humphreys and Peter Harris, NPL
% Created        24-07-2013   From Qgen_compact.m to avoid forming full 
%                             covariance matrix.
% Last amended   26-03-2014   Added input/output parameters.
%                08-04-2014   Removed matrix-matrix product.
% Released       30-06-2014
% -------------------------------------------------------------------------
% Create compact representation of covariance matrix defined by a 
% specific number of eigenvalues (principal components) without storing
% the full covariance matrix.
%
% See: Guidance for the use of compact covariance uncertainty software,
%      David Humphreys and Peter Harris, NPL REPORT TQE 10, 2014
% -------------------------------------------------------------------------
% Input:
%    nei         1 x 1        Number of eigenvalues.
%    pname      string        Pathname to folder containing harmonic
%                             results.
%    n           1 x 1        Number of harmonics (excluding DC).
%    VCC0,
%    VSS0,
%    VCS0        n x 1        Diagonals of full covariance matrix.
%
% Output:
%    Qc,
%    Qs          n x nei
%    D           n x 3        Submatrices defining compact representation.
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Evaluate the nei largest eigenvalues and corresponding eigenvectors.
%
  opts.issym = 1;
  [Q, E] = eigs(@(x)CVfun_compact(x,pname,n), 2*n, nei, 'lm', opts);
%
% Form output: (i) eignevectors scaled by the square roots of the 
% eigenvalues stored as the sub-matrices Qc and Qs corresponding to the
% Re and Im parts; (ii) differences between the "diagonals" of the full
% and approximate covariance matrices for the Re/Re, Im/Im and Im/Re parts. 
%
  Qc = Q(1:n,:)*sqrt(E);
  Qs = Q(1+n:2*n,:)*sqrt(E);
  D  = zeros(n,3);
  for k = 1:n
     Qck    = Qc(k,:);
     Qsk    = Qs(k,:);
     D(k,1) = VCC0(k) - Qck*Qck';
     D(k,2) = VCS0(k) - Qsk*Qck';
     D(k,3) = VSS0(k) - Qsk*Qsk';
  end
%
% -------------------------------------------------------------------------
% End.
%
% Copyright NPL. No warranty or guarantee applies to this software, and 
% therefore any users should satisfy themselves that it meets their 
% requirements.
% -------------------------------------------------------------------------