function [z, zref] = alignment(z)
% -------------------------------------------------------------------------
% alignment.m
%
% Author         David Humphreys and Peter Harris, NPL
% Created        07-06-2012
% Last amended   07-06-2012
% Released       08-02-2013
% Last amended   26-03-2014   Added input/output parameters.
% Released       30-06-2014
% -------------------------------------------------------------------------
% Alignment of repeated harmonic results.
%
% See: Guidance for the use of compact covariance uncertainty software,
%      David Humphreys and Peter Harris, NPL REPORT TQE 10, 2014
% -------------------------------------------------------------------------
% Input:
%    z           n x m        Complex array storing m repeated measurements
%                             of n harmonic results before alignment.
%
% Output:
%    z           n x m        Complex array storing m repeated measurements
%                             of n harmonic results after alignment.
%    zref        n x 1        Complex array storing averages of n harmonic
%                             results.
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Number n of frequencies (excluding DC) and number m of waveforms.
%
  [n, m] = size(z);
%
% Calculate average delay correction for each waveform.
%
  zref = mean(z,2);
  dz   = zeros(1,m);
  for j = 1:m
     dz1   = z(2:n,j).*conj(z(1:n-1,j));
     dzr   = zref(2:n).*conj(zref(1:n-1));
     dzn   = conj(dz1).*dzr;
     dz(j) = sum(dzn)/abs(sum(dzn));
  end
%
% Apply correction, and evaluate the mean.
%
  for i = 1:n
     z(i,:) = z(i,:).*(dz.^i);
  end
  zref = mean(z,2);
%
% Calculate the delay for the mean of the corrected harmonic results.
%
  dz0n = conj(zref(2:n)).*zref(1:n-1);
  dz0  = sum(dz0n)/abs(sum(dz0n));
%
% Align the results so that the mean has zero delay.
%
  for i = 1:n
     z(i,:) = z(i,:)*(dz0^i);
  end
  zref = mean(z,2);
%
% -------------------------------------------------------------------------
% End.
%
% Copyright NPL. No warranty or guarantee applies to this software, and 
% therefore any users should satisfy themselves that it meets their 
% requirements.
% -------------------------------------------------------------------------